@@ -38,7 +38,7 @@
 # define N_(String) (String)
 #endif
 
-/*#define GVM_DEBUG*/
+#define GVM_DEBUG
 #ifdef GVM_DEBUG
 # define dbg(fmt,arg...) fprintf(stderr, "%s/%d: " fmt,__FILE__,__LINE__,##arg)
 #else
@@ -47,15 +47,13 @@
 
 #define warn(fmt,arg...) g_warning("%s/%d: " fmt,__FILE__,__LINE__,##arg)
 
-#define BIN_MOUNT		"/bin/mount"	/* what we mount with */
-#define BIN_UMOUNT		"/bin/umount"	/* what we unmount with */
-#define NAUTILUS_COMMAND	"/usr/bin/nautilus -n --no-desktop %m"
+#define NAUTILUS_COMMAND	 BIN_NAUTILUS" -n --no-desktop %m"
 
 static struct gvm_configuration config;
 static LibHalContext *hal_ctx;
 
-/** List of UDI's for volumes mounted when starting up */
-static GSList *mount_ignore_udi_list = NULL;
+/** List of UDI's for volumes mounted by g-v-m that we need to apply policy to*/
+static GSList *mounted_volumes_policy_queue = NULL;
 
 /** List of UDI's of all volumes mounted during the lifetime of the program */
 static GSList *all_mounted_volumes = NULL;
@@ -217,7 +215,7 @@ error:
 /*
  * gvm_ask_autorun - ask the user if they want to autorun a specific file
  *
- * Returns TRUE if the user selected 'Yes' and FALSE otherwise
+ * Returns TRUE if the user selected 'Run' and FALSE otherwise
  */
 static gboolean
 gvm_ask_autorun (const char *path)
@@ -225,16 +223,26 @@ gvm_ask_autorun (const char *path)
 	GtkWidget *askme;
 	gboolean retval;
 
-	askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION,
-					GTK_BUTTONS_YES_NO,
-					_("Do you want to run %s?"), path);
-	gtk_dialog_set_default_response (GTK_DIALOG (askme), GTK_RESPONSE_YES);
+	askme = gtk_message_dialog_new (NULL,
+					0, GTK_MESSAGE_WARNING,
+					GTK_BUTTONS_NONE,
+					_("Run command from inserted media?"));
+	/*FIXME!! A better secondary label....*/
+	gtk_message_dialog_format_secondary_text (GTK_DIALOG (askme),
+			_("The file \"%s\" on the inserted media is an auto-run"
+			  "file."));
+
+	gtk_dialog_add_buttons (GTK_DIALOG (askme),
+				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+				_("_Run Command"), GTK_RESPONSE_ACCEPT,
+				NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (askme), GTK_RESPONSE_ACCEPT);
 
 	switch (gtk_dialog_run (GTK_DIALOG (askme))) {
-	case GTK_RESPONSE_YES:
+	case GTK_RESPONSE_ACCEPT:
 		retval = TRUE;
 		break;
-	case GTK_RESPONSE_NO:
+	case GTK_RESPONSE_REJECT:
 	default:
 		retval = FALSE;
 		break;
@@ -275,10 +283,6 @@ gvm_check_dvd (const char *device, const char *mount_point, const char *udi)
 	return retval;
 }
 
-#define ASK_PHOTOS_MSG	"There are photographs on this device.\n\n" \
-			"Would you like to import these photographs " \
-			"into your photo album?"
-
 /*
  * gvm_check_photos - check if this device is a digital camera or a storage
  * unit from a digital camera (e.g., a compact flash card).  If it is, then
@@ -310,13 +314,18 @@ gvm_check_photos (const char *udi, const char *device, const char *mount_point)
 		warn ("failed to set content.photos on %s\n", device);
 
 	if (config.autophoto) {
-		askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION,
+		askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_WARNING,
 						GTK_BUTTONS_NONE,
-						_(ASK_PHOTOS_MSG));
+						_("Import photos from device?"));
+		gtk_message_dialog_format_secondary_text (GTK_DIALOG (askme),
+							  _("There are photos on the inserted media. "
+							    "Would you like to import these "
+							    "photos into your photo album?"));
+
 		gtk_dialog_add_buttons (GTK_DIALOG (askme),
-					GTK_STOCK_CANCEL,
-					GTK_RESPONSE_CANCEL, _("_Import"),
-					IMPORT, NULL);
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					_("_Import Photos"), IMPORT,
+					NULL);
 		action = gtk_dialog_run (GTK_DIALOG (askme));
 		gtk_widget_destroy (askme);
 
@@ -336,7 +345,6 @@ out:
  * ask the user if he wants to import the photos.
  *
  * Returns TRUE if this was a camera, FALSE otherwise
- *
  */
 static gboolean
 gvm_check_camera (const char *udi)
@@ -353,13 +361,17 @@ gvm_check_camera (const char *udi)
 	dbg ("Camera detected: %s\n", udi);
 
 	if (config.autophoto) {
-		askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION,
+		askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_WARNING,
 						GTK_BUTTONS_NONE,
-						_(ASK_PHOTOS_MSG));
+						_("Import photos from camera?"));
+		gtk_message_dialog_format_secondary_text (GTK_DIALOG (askme),
+							  _("There are photos on the plugged-in camera. "
+							    "Would you like to import these "
+							    "photos into your photo album?"));
 		gtk_dialog_add_buttons (GTK_DIALOG (askme),
-					GTK_STOCK_CANCEL,
-					GTK_RESPONSE_CANCEL, _("_Import"),
-					IMPORT, NULL);
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					_("_Import Photos"), IMPORT,
+					NULL);
 		action = gtk_dialog_run (GTK_DIALOG (askme));
 		gtk_widget_destroy (askme);
 
@@ -517,10 +529,6 @@ gvm_run_cdplayer (const char *device, const char *mount_point, const char *udi)
 				 mount_point, udi);
 }
 
-#define ASK_MIXED_MSG	"This CD has both audio tracks and data files.\n" \
-			"Would you like to play the audio tracks or " \
-			"browse the data files?"
-
 /*
  * gvm_ask_mixed - if a mixed mode CD (CD Plus) is inserted, we can either
  * mount the data tracks or play the audio tracks.  How we handle that depends
@@ -543,13 +551,21 @@ gvm_ask_mixed (const char *udi)
 		GtkWidget *askme;
 
 		askme = gtk_message_dialog_new (NULL, 0,
-						GTK_MESSAGE_QUESTION,
+						GTK_MESSAGE_WARNING,
 						GTK_BUTTONS_NONE,
-						_(ASK_MIXED_MSG));
+						_("Browse files or play tracks from disc?"));
+
+		gtk_message_dialog_format_secondary_text (GTK_DIALOG (askme),
+							  _("There are both audio tracks and "
+							    "data files on the inserted disc. "
+							    "Choose if you want to play the audio "
+							    "tracks to listen to music or browse "
+							    "the files to manage the stored data."));
 		gtk_dialog_add_buttons (GTK_DIALOG (askme),
-					GTK_STOCK_CANCEL,
-					GTK_RESPONSE_CANCEL, _("_Play"),
-					PLAY, _("_Browse"), MOUNT, NULL);
+					_("_Browse Files"), MOUNT,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					_("_Play Tracks"), PLAY,
+					NULL);
 		action = gtk_dialog_run (GTK_DIALOG (askme));
 		gtk_widget_destroy (askme);
 	} else if (config.automount_media)
@@ -560,6 +576,7 @@ gvm_ask_mixed (const char *udi)
 	switch (action) {
 	case MOUNT:
 		gvm_device_mount (device);
+		mounted_volumes_policy_queue = g_slist_append (mounted_volumes_policy_queue, g_strdup (udi));
 		break;
 	case PLAY:
 		gvm_run_cdplayer (device, device, udi);
@@ -634,8 +651,10 @@ gvm_cdrom_policy (const char *udi)
 	} else if (has_audio && has_data) {
 		gvm_ask_mixed (udi);
 	} else if (has_data) {
-		if (config.automount_media)
+		if (config.automount_media) {
 			gvm_device_mount (device);
+			mounted_volumes_policy_queue = g_slist_append (mounted_volumes_policy_queue, g_strdup (udi));
+		}
 	} else if (is_blank) {
 		if (gvm_device_is_writer (drive_udi))
 			gvm_run_cdburner (device, device, udi);
@@ -659,10 +678,21 @@ out:
  * At the moment, we only handle CD-ROM and DVD drives.
  */
 static void
-gvm_media_changed (const char *udi, const char *storage_device)
+gvm_media_changed (const char *udi, const char *storage_device, 
+		   const char *device)
 {
 	char *media_type;
 
+	/* Refuse to enforce policy on removable media if drive is locked */
+	if (hal_device_property_exists (
+		    hal_ctx, storage_device, "info.locked") &&
+	    hal_device_get_property_bool (
+		    hal_ctx, storage_device, "info.locked")) {
+		dbg ("Drive with udi %s is locked through hal; "
+		     "skipping policy\n", storage_device);
+		return;
+	}
+
 	/*
 	 * Get HAL's interpretation of our media type.  Note that we must check
 	 * the storage device and not this UDI
@@ -674,10 +704,16 @@ gvm_media_changed (const char *udi, const char *storage_device)
 		return;
 	}
 
-	if (!g_strcasecmp (media_type, "cdrom"))
+	if (!g_strcasecmp (media_type, "cdrom")) {
 		gvm_cdrom_policy (udi);
-
-	/* other media_types go here */
+	} else {
+		dbg ("Added: %s\n", device);
+		
+		if (config.automount_drives) {
+			gvm_device_mount (device);
+			mounted_volumes_policy_queue = g_slist_append (mounted_volumes_policy_queue, g_strdup (udi));
+		}
+	}
 
 	hal_free_string (media_type);
 }
@@ -730,15 +766,17 @@ hal_device_added (LibHalContext *ctx __attribute__((__unused__)),
 					  "storage.removable")) {
 		/* we handle media change events separately */
 		dbg ("Changed: %s\n", device);
-		gvm_media_changed (udi, storage_device);
+		gvm_media_changed (udi, storage_device, device);
 		goto out;
 	}
 	
 	/* folks, we have a new device! */
 	dbg ("Added: %s\n", device);
 	
-	if (config.automount_drives)
+	if (config.automount_drives) {
 		gvm_device_mount (device);
+		mounted_volumes_policy_queue = g_slist_append (mounted_volumes_policy_queue, g_strdup (udi));
+	}
 	
 out:
 	hal_free_string (device);
@@ -806,7 +844,7 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 	
 	val = hal_device_get_property_bool (hal_ctx, udi, key);
 	if (val == TRUE) {
-		gboolean ignore_mount;
+		GSList *policy_udi;
 
 		dbg ("Mounted: %s\n", udi);
 
@@ -814,26 +852,15 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 		all_mounted_volumes = g_slist_append (all_mounted_volumes,
 						      g_strdup (udi));
 
-		ignore_mount = FALSE;
-		for (i = mount_ignore_udi_list; i != NULL; i = next) {
-			next = g_slist_next (i);
-
-			if (strcmp (udi, (const char *)i->data) == 0) {
-				ignore_mount = TRUE;
-				g_free (i->data);
-				mount_ignore_udi_list = 
-					g_slist_delete_link (
-						mount_ignore_udi_list, i);
-				break;
-			}
-		}
-		
-		
-		/* only autorun if not in ignore list, cf.
-		 * function mount_all
-			 */
-		if (!ignore_mount)
+		policy_udi = g_slist_find_custom (mounted_volumes_policy_queue, 
+						  udi, 
+						  g_ascii_strcasecmp);
+		if (policy_udi != NULL) {
+			g_free (policy_udi->data);
+			mounted_volumes_policy_queue = g_slist_delete_link (mounted_volumes_policy_queue, 
+									    policy_udi);
 			gvm_device_autorun (udi);
+		}
 
 	} else {
 		dbg ("Unmounted: %s\n", udi);
@@ -952,9 +979,10 @@ static void mount_all (LibHalContext *ctx)
 
 		/* only mount if the block device got a sensible filesystem */
 		if (!hal_device_property_exists (ctx, udi, 
-						"volume.is_filesystem") ||
-		    !hal_device_get_property_bool (ctx, udi, 
-						   "volume.is_filesystem"))
+						"volume.fsusage") ||
+		    strcmp (hal_device_get_property_string (ctx, udi, 
+							    "volume.fsusage"), 
+			    "filesystem") != 0)
 			continue;
 
 		device_file = hal_device_get_property_string (ctx, udi,
@@ -964,29 +992,18 @@ static void mount_all (LibHalContext *ctx)
 
 			dbg ("mount_all: mounting %s\n", device_file);
 
-			if (gvm_device_mount (device_file)) {
-				/* yay, it worked; add to list of
-				 * udi's not to autorun from hal_property_
-				 * changed
-				 */
-
-				mount_ignore_udi_list = 
-					g_slist_append (mount_ignore_udi_list,
-							g_strdup (udi));
-
-			}
+			gvm_device_mount (device_file);
 
 			hal_free_string (device_file);
-		} else {
+		} else
 			warn ("no device_file for udi=%s\n", udi);
-		}
 	}
 
 	hal_free_string_array (volumes);
 }
 
-/* Unmount all volumes that were mounted during the lifetime of this
- * g-v-m instance
+/** Unmount all volumes that were mounted during the lifetime of this
+ *  g-v-m instance
  *
  *  @param  ctx                 LibHal context
  */
