@@ -369,14 +369,15 @@ init_capplet (void)
 	if (!check_if_hal_is_running ()) {
 		GtkWidget *dialog;
 
-		dialog = gtk_message_dialog_new_with_markup (
-			NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-			"<span weight=\"bold\" size=\"larger\">"
-			"Volume Management not Supported"
-			"</span>\n\n"
-			"The \"hald\" service is required but not currently "
-			"running. Enable the service and rerun this applet, "
-			"or contact your system administrator.");
+		dialog = gtk_message_dialog_new (NULL,
+						 0, GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("Volume management not supported"));
+		gtk_message_dialog_format_secondary_text (GTK_DIALOG (dialog),
+							  _("The \"hald\" service is required but not currently "
+							    "running. Enable the service and rerun this applet, "
+							    "or contact your system administrator."));
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		exit (EXIT_FAILURE);
@@ -396,17 +397,13 @@ init_capplet (void)
 		dialog = gtk_message_dialog_new (NULL,
 						 0, GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_OK,
-						 "<b>%s</b>\n%s.",
 						 _("Could not load the main "
-						   "interface"),
-						 _("Please make sure that the "
-						   "volume manager is properly "
-						   "installed"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-		gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-		gtk_label_set_use_markup (GTK_LABEL
-					  (GTK_MESSAGE_DIALOG (dialog)->label),
-					  TRUE);
+						   "interface"));
+		gtk_message_dialog_format_secondary_text (GTK_DIALOG (dialog),
+				 _("Please make sure that the "
+				   "volume manager is properly "
+				   "installed"));
+
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 						 GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -419,22 +416,17 @@ init_capplet (void)
 	icon_theme = gnome_icon_theme_new ();
 
 	icon = glade_xml_get_widget (xml, "removable_drives_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-zipdisk");
+	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-removable");
 	icon = glade_xml_get_widget (xml, "audio_cd_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-cdrom-audio");
 	icon = glade_xml_get_widget (xml, "blank_cd_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-cdrom");
+	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-disc-cdr");
 	icon = glade_xml_get_widget (xml, "dvd_video_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-dvd");
 	icon = glade_xml_get_widget (xml, "digital_camera_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "applets-screenshooter");
 
-	file = gnome_icon_theme_lookup_icon (icon_theme,
-			"gnome-dev-cdrom", 48, NULL, NULL);
-	if (file) {
-		gtk_window_set_default_icon_from_file (file, NULL);
-		g_free (file);
-	}
+	gtk_window_set_default_icon_name ("gnome-dev-cdrom");
 
 	g_object_unref (icon_theme);
 
